local addon = LibStub("AceAddon-3.0"):NewAddon("Battleground", "AceEvent-3.0", "LibSink-2.0")
addon:SetDefaultModuleState(false)

addon.landmarks = BG_LANDMARKS
local landmarks = BG_LANDMARKS

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

-- evt funcs
local CheckZone
local AnalyzeLandmarks
local AnalyzeTimeElapsed
local Message
local AnalyzeMembers
local AnalyzeTimeElapsed
local CheckTimers

-- random locals
local pois = {}
local oldzone = nil
local starttimers = {
	[L["2 minute"]] = 122,
	[L["1 minute"]] = 62,
	[L["One minute until"]] = 62,
	[L["60 seconds"]] = 62,
	[L["30 seconds"]] = 31,
	[L["15 seconds"]] = 16,
	[L["Forty five seconds"]] = 46,
	[L["Thirty seconds until"]] = 31,
	[L["Fifteen seconds until"]] = 16,
}
local captureTimer = 0
local startTimer = 122

-- gui locals
local frame
local namedButtons = {}
addon.namedButtons = namedButtons
local availableButtons = {}
local getButton
local createGUI
local showGUI
local hideGUI
local positionButton
local OnUpdateCountDown
local OnUpdateCountUp
local updateButton
local stateRows = 2
local hordecounter
local alliancecounter

-- database local ref
local db

local defaults = {
	profile = {
		sink20OutputSink = "RaidWarning",
	}
}

local options = nil
local function giveOptions()
	if not options then
		options = {
			type = "group",
			name = L["Battleground"],
			get = function( k ) return db[k.arg] end,
			set = function( k, v ) db[k.arg] = v end,
			args = {
				header = {
					type = "header",
					name = L["Message Output"],
					order = 9,
				},
				desc = {
					type = "description",
					name = L["You can select where you want messages displayed using the options below."],
					order = 10,
				},
				sink = addon:GetSinkAce3OptionsDataTable(),
			}
		}
		options.args.sink.inline = true
		options.args.sink.name = ""
	end
	return options
end


function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BattlegroundDB", defaults, "Default")
	self:SetSinkStorage(self.db.profile)
	db = self.db.profile

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Battleground", giveOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Battleground", L["Battleground"])
end

function addon:OnEnable()
	self:RegisterMessage("Battleground_Message", Message)
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", CheckZone)
	CheckZone()
end

function AnalyzeLandmarks()
	for i = 1, GetNumMapLandmarks(), 1 do
		local name, desc, ti, x, y = GetMapLandmarkInfo(i)
		local old = pois[name]
		if landmarks[ti] and ( not old or old ~= ti ) then
			pois[name] = ti
			local button = updateButton(name, ti)
			-- print("BG_P", name, unpack(landmarks[ti]))
			addon:SendMessage("Battleground_POIUpdate", name, desc, ti, button)
		end
	end
end

local startChecker = CreateFrame("Frame", nil, UIParent)
startChecker:Hide()
startChecker:SetScript("OnUpdate", function(self, elapsed)
	if not self.elapsed or self.elapsed > 1 then 
		self.elapsed = 0
		AnalyzeTimeElapsed(startTimer)
	end
	self.elapsed = self.elapsed + elapsed
end)

function AnalyzeTimeElapsed(offset)
	if not namedButtons[L["Battleground Start"]] then return end
	local start = floor(GetBattlefieldInstanceRunTime()/1000)
	if start < offset then
		local poi = namedButtons[L["Battleground Start"]]
		poi.text:SetText(L["Battleground Start"])
		poi.switchname = L["Battleground Elapsed"]
		if start == 0 then
			startChecker:Show() -- damn fucking cockbollocks
		else 
			startChecker:Hide()
		end
		poi.expire = time() + (offset - start)
		poi:SetScript("OnUpdate", OnUpdateCountDown)
		addon:SendMessage("Battleground_StartTimer", offset - start)
	else
		startChecker:Hide()
		local poi = namedButtons[L["Battleground Start"]]
		poi.expire = time() - (start - offset)
		poi.text:SetText(L["Battleground Elapsed"])
		poi:SetScript("OnUpdate", OnUpdateCountUp)
		addon:SendMessage("Battleground_Started", start - offset)
	end
end

function CheckTimers(event, msg)
	if not BattlegroundDB then BattlegroundDB = {} end
	if not BattlegroundDB.log then BattlegroundDB.log = {} end
	if not BattlegroundDB.log[event] then BattlegroundDB.log[event] = {} end
	BattlegroundDB.log[event][msg] = true

	local runtime = floor(GetBattlefieldInstanceRunTime()/1000)
	for txt, t in pairs(starttimers) do
		if msg:match(txt) then
			local timeShouldHavePassed = startTimer - t
			startTimer = startTimer + (runtime - timeShouldHavePassed)
			AnalyzeTimeElapsed(startTimer)
			break
		end
	end
	if msg:match(L["has begun!"]) or msg:match(L["begins!"]) then
		startTimer = runtime
		AnalyzeTimeElapsed(startTimer) -- our offset is the current runtime
	end
end

function ChatForward(event, msg)
	addon:SendMessage("Battleground_Message", msg, ChatTypeInfo[strsub(event, 10)])
	if event == "CHAT_MSG_BG_SYSTEM_NEUTRAL" then
		CheckTimers(event, msg)
	end
end

function AnalyzeMembers()
	if not AlwaysUpFrame2 then return end
	
	local alliance, horde =  0, 0
	for i = 1, GetNumBattlefieldScores(), 1 do
		local faction = select(6 , GetBattlefieldScore(i))
		if faction == 0 then horde = horde + 1 
		elseif faction == 1 then alliance = alliance + 1 end
	end
	if not alliancecounter then
		alliancecounter = AlwaysUpFrame1:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
		hordecounter = AlwaysUpFrame2:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	end
	local offset = ((not AlwaysUpFrame1Icon:GetTexture() or AlwaysUpFrame1Icon:GetTexture() == "") and 1) or 0
	alliancecounter:SetPoint("RIGHT", _G["AlwaysUpFrame"..(1+offset).."Icon"], "LEFT", 0, 10)
	hordecounter:SetPoint("RIGHT", _G["AlwaysUpFrame"..(2+offset).."Icon"], "LEFT", 0, 10)
	alliancecounter:SetText(alliance)
	hordecounter:SetText(horde)
end

local scorerequestor = CreateFrame("Frame")
scorerequestor:Hide()
scorerequestor:SetScript("OnUpdate", function(self, elapsed)
	if not self.elapsed or self.elapsed > 5 then
		self.elapsed = 0
		RequestBattlefieldScoreData()
	end
	self.elapsed = self.elapsed + elapsed
end)

function CheckZone()
	if oldzone then
		wipe(pois)
		for name, module in addon:IterateModules() do module:Disable() end
		hideGUI()
		oldzone = nil
		addon:UnregisterEvent("WORLD_MAP_UPDATE")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE")
		addon:UnregisterEvent("UPDATE_BATTLEFIELD_SCORE")
		scorerequestor:Hide()
	end
	SetMapToCurrentZone()
	local zone = GetMapInfo()
	local module = addon:GetModule(zone, true)
	if module then
		captureTimer = module.captureTimer
		startTimer = module.startTimer or 122
		oldzone = zone
		staterows = module.stateRows or 2
		showGUI(module)
		module:Enable()
		addon:RegisterEvent("WORLD_MAP_UPDATE", AnalyzeLandmarks)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL", ChatForward)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", ChatForward)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", ChatForward)
		addon:RegisterEvent("UPDATE_BATTLEFIELD_SCORE", AnalyzeMembers)
		AnalyzeLandmarks()
		AnalyzeTimeElapsed(startTimer) -- default to 2 minutes
		scorerequestor:Show()
	end
end

function Message(message, text, color, chatFrame)
	if not text then return end
	local r, g, b = 1, 1, 1
	if type(color) == "string" then
		if color == FACTION_ALLIANCE then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		elseif color == FACTION_HORDE then color = ChatTypeInfo["BG_SYSTEM_HORDE"] end
	end
	if type(color) == "table" and color.r and color.g and color.b then
		r, g, b = color.r, color.g, color.b
	end
	addon:Pour(text, r, g, b)
	if chatFrame then
		chatFrame:AddMessage(text, r, g, b)
	end
end

local function onClick(self, button)
	if button == "LeftButton" and IsShiftKeyDown() then
		if self.expire and self.status == "horde_assaulted" or self.status == "alliance_assaulted" then
			local faction = self.status == "horde_assaulted" and FACTION_HORDE or FACTION_ALLIANCE
			local tleft = self.expire - time()
			local m = floor(tleft/60)
			local s = tleft % 60
			local msg = string.format("%s: %s %0d:%02d", faction, self.text:GetText(), m, s)
			SendChatMessage(msg, "BATTLEGROUND")
		end
	end
end

function getButton(location)
	local text, time, icon
	local button = tremove(availableButtons)
	if not button then
		button = CreateFrame("Button", nil, frame)
		text = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
		time = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
		icon = button:CreateTexture(nil, "BACKGROUND")
		button:SetScript("OnClick", onClick)
		button:EnableMouse(true)
		button:SetWidth(16)
		button:SetHeight(16)
		text:SetWidth(200)
		text:SetHeight(16)
		time:SetWidth(200)
		time:SetHeight(16)
		button.text = text
		button.time = time
		button.icon = icon
		icon:SetPoint("CENTER", button, "CENTER")
		icon:SetWidth(16)
		icon:SetHeight(16)
		button:SetHeight(16)
	end
	text = button.text
	icon = button.icon
	time = button.time
	time:ClearAllPoints()
	text:ClearAllPoints()
	text:SetTextColor(1,1,1)
	time:SetTextColor(1,1,1)
	button:SetAlpha(1)
	icon:SetTexture("Interface\\Minimap\\POIIcons")
	if location == "left" then
		button.align = "left"
		text:SetPoint("RIGHT", button, "LEFT")
		time:SetPoint("LEFT", button, "RIGHT")
		text:SetJustifyH("RIGHT")
		time:SetJustifyH("LEFT")
	else
		button.align = "right"
		time:SetPoint("RIGHT", button, "LEFT")
		text:SetPoint("LEFT", button, "RIGHT")
		time:SetJustifyH("RIGHT")
		text:SetJustifyH("LEFT")
	end
	button:Show()
	return button
end

function createGUI()
	if frame then return end
	frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	frame:SetPoint("TOP", WorldStateAlwaysUpFrame, "TOP", 30, -8) -- -66)
	frame:SetWidth(260)
	frame:SetHeight(30)
end

function positionButton(location, button, last)
	if not last then
		if location == "left" then
			button:SetPoint("RIGHT", frame, "LEFT")
		elseif location == "right" then
			button:SetPoint("LEFT", frame, "RIGHT")
		elseif location == "top" then
			button:SetPoint("BOTTOM", frame, "TOP", -50, 0)
		else -- bottom
			button:SetPoint("TOP", frame, "TOP", -50, -8 - (24*stateRows)) ---56) 
		end
	elseif location == "right" or location == "left" or location == "bottom" then
		button:SetPoint("TOP", last, "BOTTOM")
	else -- top
		button:SetPoint("BOTTOM", last, "TOP")
	end
end

function showGUI(module)
	createGUI()
	for location, buttons in pairs(module.buttons) do
		local last = nil
		for k, name in ipairs(buttons) do
			local button = getButton(location)
			namedButtons[name] = button
			button.text:SetText(name)
			positionButton(location, button, last)
			last = button
		end
	end
	-- damn exceptions
	if namedButtons[L["Battleground Start"]] then
		namedButtons[L["Battleground Start"]].icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
		namedButtons[L["Battleground Start"]].icon:SetTexCoord(0.57, 0.90, 0.08, 0.41)
	end
	if namedButtons[L["Bases to win: %d"]] then
		namedButtons[L["Bases to win: %d"]].icon:SetTexCoord(landmarks[6][3], landmarks[6][4], landmarks[6][5], landmarks[6][6])
		namedButtons[L["Bases to win: %d"]]:Hide()
	end
	if namedButtons[L["Final Score: %d/%d"]] then
		namedButtons[L["Final Score: %d/%d"]].icon:SetTexCoord(landmarks[6][3], landmarks[6][4], landmarks[6][5], landmarks[6][6])
		namedButtons[L["Final Score: %d/%d"]]:Hide()
	end
	frame:Show()
end

function hideGUI()
	for k, button in pairs(namedButtons) do
		button:Hide()
		button:ClearAllPoints()
		button:SetScript("OnUpdate", nil)
		button:SetScript("OnClick", nil)
		tinsert(availableButtons, button)
	end
	if alliancecounter then
		alliancecounter:SetText("")
		hordecounter:SetText("")
	end
	frame:Hide()
	wipe(namedButtons)
end

function OnUpdateCountUp(self, elapsed)
	if self.expire then
		local telapsed = time() - self.expire
		local m = floor(telapsed/60)
		local s = telapsed % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	end
end
addon.OnUpdateCountUp = OnUpdateCountUp

function OnUpdateCountDown(self, elapsed)
	if not self.expire then return end
	if self.expire >= time() then
		local tleft = self.expire - time()
		local m = floor(tleft/60)
		local s = tleft % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	elseif self.switchname then
		self.text:SetText(self.switchname)
		self.switchname = nil
		self.expire = time()
		self:SetScript("OnUpdate", OnUpdateCountUp)
	end
end
addon.OnUpdateCountDown = OnUpdateCountDown

function updateButton(name, ti)
	if not namedButtons[name] then return end
	local poi = namedButtons[name]
	poi.icon:SetTexCoord( landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	local status = landmarks[ti][2]
	poi:SetAlpha(1)
	poi.text:SetTextColor(.6,.6,.6)
	
	if status == "horde" then
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	elseif status == "alliance" then
		local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	end
	if ( status == "alliance_assaulted" or status == "horde_assaulted" ) and poi.status then
		poi.expire = time() + captureTimer
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		if status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
		poi.time:SetTextColor(color.r, color.g, color.b)
		poi:SetScript("OnUpdate", OnUpdateCountDown)
	else
		poi.expire = nil
		poi:SetScript("OnUpdate", nil)
		poi.time:SetText(nil)
		poi.charName = nil
	end
	poi.status = status
	return poi
end
